import React from "react";
import { FlatList, View, Text, StyleSheet, TouchableOpacity, ActivityIndicator } from "react-native";
import {Quiz} from "../../models/Quiz";
import { NavigationProp } from "@react-navigation/native";



/**
 * QuizList : Un composant affichant une liste de quiz avec leur titre, nombre de questions, et catégorie.
 *
 * @component
 *
 * @param navigation - Propriété de navigation utilisée pour naviguer vers l'écran "InformationsOfQuiz".
 * @param quizList - Liste des quiz à afficher. Peut être null ou undefined.
 * @param isLoadingData - Boolean indiquant si les données sont en cours de chargement.
 *
 * @example
 * <QuizList
 *     navigation={navigation}
 *     quizList={quizzes}
 *     isLoadingData={isLoading}
 * />
 *
 * @description
 * - Ce composant gère trois états :
 *   1. Chargement : Affiche un spinner si `isLoadingData` est true.
 *   2. Liste des quiz : Affiche une liste interactive des quiz si `quizList` contient des éléments.
 *   3. Liste vide : Affiche un message si `quizList` est vide.
 *
 * - Chaque quiz affiche :
 *   - Son code en tant que titre.
 *   - Son nombre total de questions.
 *   - Sa catégorie, affichée en majuscules.
 *
 * @returns Un composant React contenant une liste interactive de quiz ou des messages en fonction de l'état.
 */
interface Props {
    quizList: Quiz[] | null | undefined;
    isLoadingData: boolean;
    nextPage?: () => void;
    onQuizPressed:(quiz: Quiz) => void;
}
export default function QuizList({ quizList, isLoadingData, nextPage, onQuizPressed }: Props) {
    if (quizList) {
    }
    const renderQuizItem = ({ item }: { item: Quiz }) => (
        <TouchableOpacity style={styles.quizItem} onPress={() => onQuizPressed(item)}>
            <Text style={styles.quizTitle}>{item.name}</Text>
            <Text style={styles.quizDetails}>
                {item.questionCount} QUESTIONS | {item.difficulty.name} | {item.author.username}
            </Text>
        </TouchableOpacity>
    );

    return (
        <View style={styles.container}>
            {isLoadingData ? (
                <View style={styles.loadingContainer}>
                    <ActivityIndicator size="large" color="#2b73fe" />
                    <Text style={styles.loadingText}>Loading quizzes...</Text>
                </View>
            ) : quizList && quizList.length > 0 ? (
                <FlatList
                    data={quizList}
                    renderItem={renderQuizItem}
                    keyExtractor={(item,index) => item.id+ ""+ index}
                    showsVerticalScrollIndicator={true}
                    contentContainerStyle={styles.listContent}
                    onEndReached={() => {
                        if (nextPage) {
                            nextPage();
                        }
                    }}

                    onEndReachedThreshold={0.5}
                />
            ) : (
                <Text style={styles.emptyMessage}>No quizzes available.</Text>
            )}
        </View>
    );
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        backgroundColor: "white",
        padding: 10,
        borderRadius: 10,
        borderColor: "#fafafa",
        borderWidth: 3,
        elevation: 5, // Ombre pour Android
        shadowColor: "#000", // Ombre pour iOS
        shadowOffset: { width: 0, height: 4 },
        shadowOpacity: 0.25,
        shadowRadius: 3.84,
        width: "100%", // Assurez-vous que le conteneur occupe toute la largeur
    },
    listContent: {
    },
    quizItem: {
        backgroundColor: "#f3f3f3",
        padding: 15,
        borderRadius: 10,
        marginVertical: 5,
        elevation: 3, // Ombre pour Android
        shadowColor: "#000", // Ombre pour iOS
        shadowOffset: { width: 0, height: 2 },
        shadowOpacity: 0.2,
        shadowRadius: 2,
        width: "100%",
        alignSelf: "center", // Centre les éléments dans leur conteneur
    },
    quizTitle: {
        fontSize: 22,
        fontWeight: "bold",
        color: "#000",
        marginBottom: 5,
        textAlign: "center",
    },
    quizDetails: {
        fontSize: 18,
        color: "#555",
        textAlign: "center",
    },
    emptyMessage: {
        fontSize: 18,
        color: "#888",
        textAlign: "center",
        marginTop: 20,
    },
    loadingContainer: {
        flex: 1,
        justifyContent: "center",
        alignItems: "center",
    },
    loadingText: {
        fontSize: 16,
        color: "#555",
        marginTop: 10,
        textAlign: "center",
    },
});
