import {View, Text, StyleSheet, Image, FlatList} from "react-native";
import {NavigationProp, RouteProp} from "@react-navigation/native";
import React, {useEffect, useState} from "react";
import { TouchableOpacity } from "react-native";
import {User} from "../../../models/User";
import {io, Socket} from "socket.io-client";
import useCookie from "../../../hooks/UseCookie";
import {useMultiService} from "../../../services/MultiService";
import HttpError from "../../../services/HttpError";
import TemplateMenu from "../../../templates/TemplateMenu";
import BlueButton from "../../../components/buttons/BlueButton";
import WhiteButton from "../../../components/buttons/WhiteButton";
import ModalInvitePlayer from "../../../components/Multiplayer/ModalInvitePlayer";
import EventSource from "react-native-sse";
import {CustomEventType} from "../../../models/CustomEventType";
import {Question} from "../../../models/Question";
interface Props {
    navigation: NavigationProp<any>;
    route: RouteProp<RoutePropsType, "Lobby">;
}

type RoutePropsType = {
    Lobby: {
        quizId?: string;
        roomId?: string;
        nbPlayers: number;
        isHost: boolean;
        questionCount: number;
    };
};

interface GenerateRunData {
    runId: string; // ou number selon votre cas
    // autres propriétés si nécessaire
}


export default function Lobby({navigation, route}: Props) {
    const { quizId, roomId, nbPlayers, isHost, questionCount} = route.params;

    const [players, setPlayers] = useState<User[]>([{ id: 0, username: 'Invite', email: 'invite@email.com'}]);
    const [socket, setSocket] = useState<Socket | null>(null);
    const [roomIdCreated, setRoomIdCreated] = useState<string | undefined>(roomId);
    const [runId, setRunId] = useState<string>("");
    const [showModal, setShowModal] = useState<boolean>(false);
    const [esUseState, setEsUseState] = useState<EventSource<CustomEventType>>();

    const {createParty, generateRun, startParty} = useMultiService();

    const userInvite = { id: 0, username: 'Invite', email: 'invite@email.com'}
    let goodId = roomId;
    const initSSE = async () => {
        if(isHost && quizId) {
            const response = await createParty(quizId);
            if (HttpError.isHttpError(response)) {
                return;
            }
            goodId = response;
            setRoomIdCreated(response);
        }

        const es = new EventSource<CustomEventType>(`https://klebert-host.com:33036/party/join/${goodId}`);
        setEsUseState(es);

        es.addEventListener("gameStarted",  async (event) => {
            if(!event.data) return;
            const responseGenerate = await generateRun();
            if(HttpError.isHttpError(responseGenerate)) {
                return;
            }

            setRunId(responseGenerate);
            // navigation.navigate("PlayingQuizMultiMode", {runId: responseGenerate, questionCount:event.data.questionCount, es: es, roomId: roomId, isHost: true});
            navigation.navigate("PlayingQuizMultiMode", {runId: responseGenerate, questionCount: questionCount, es: es, roomId: roomId, isHost: isHost});
        });

        es.addEventListener("userJoined", (event) => {
            if(!event.data) return;

            const newUsers: User[] = JSON.parse(event.data);

            setPlayers((prevPlayers) => {
                const ids = new Set(prevPlayers.map((player) => player.id));
                const filteredNewUsers = newUsers.filter((user) => !ids.has(user.id));
                return [...prevPlayers, ...filteredNewUsers];
            });
        });


        es.addEventListener("error", (err) => {
            console.log("Error with SSE:", err);
        });

        useEffect(() => {
            return () => {
                es.close();
            };
        }, []);
    };

    useEffect(() => {
        initSSE();
    }, []);

    const item = ({ item }: { item: User }) => {
        return (
            <>
                {item.username === 'Invite' ?
                    <View style={styles.PlayerItem}>
                        <TouchableOpacity style={styles.imagePlayer} onPress={onInvitePressed}>
                            <Image source={require('../../../assets/add.png')}  />
                        </TouchableOpacity>
                        <Text style={styles.textPlayer}>Invite</Text>
                    </View>
                :
                    <View style={styles.PlayerItem}>
                        <Image source={require('../../../assets/ProfilBaseImage.png')} style={styles.imagePlayer} />
                        <Text style={styles.textPlayer}>{item.username}</Text>
                    </View>
                }
            </>

        )
    }


    const onLeavePressed = () => {
        navigation.navigate("TabNavigator");
    };

    const onPlayPressed = () => {
        startParty();
    };

    const onCancelPressed = () => {
        navigation.navigate("TabNavigator");
    };

    const onInvitePressed = () => {
        setShowModal(true);
    };

    return (
        <View>
            <TemplateMenu navigation={navigation} headerNavigation={true} buttonGoBack={true}>
                <View style={styles.containerLobby}>
                    <Text style={styles.title}>{'BMW OFFICIAL QUIZ (I’m leaking oil)'}</Text>
                    <View style={styles.containerPlayers}>
                        <FlatList
                            data={players}
                            renderItem={item}
                            numColumns={3}
                            keyExtractor={(item) => item.id.toString()}
                            showsVerticalScrollIndicator={true}
                        />
                    </View>
                    {isHost ?
                        <View style={styles.buttonGroup}>
                            <View style={styles.button}>
                                <BlueButton text="PLAY" onPress={onPlayPressed} isDisabled={false}/>
                            </View>
                            <View style={styles.button}>
                                <WhiteButton text="CANCEL" onPress={onCancelPressed} isDisabled={false}/>
                            </View>
                        </View>
                    :
                        <View style={styles.buttonGroup}>
                            <WhiteButton text="LEAVE" onPress={onLeavePressed} isDisabled={false}/>
                        </View>
                    }
                </View>
            </TemplateMenu>
            <ModalInvitePlayer showModal={showModal} onClosePressed={() => setShowModal(false)} roomId={roomIdCreated} />
        </View>
    )
}

const styles = StyleSheet.create({
    containerPlayers: {
        height: '65%',
        width: '100%',
    },
    PlayerItem: {
        width: '30%',
        aspectRatio: 1,
        justifyContent: 'center',
        alignItems: 'center',
        margin: 5,
    },
    imagePlayer: {
        width: '80%',
        height: '80%',
    },
    textPlayer: {
        fontSize: 20,
        textAlign: 'center',
    },
    container: {
        display: "flex",
        height: "100%",
        width: "100%",
        justifyContent: "center",
        alignItems: "center",
    },
    title: {
        fontSize: 32,
        textAlign: 'center',
        color: '#00B3F4',
        marginBottom: 20,
    },
    buttonGroup: {
        flexDirection: 'row',
        justifyContent: 'space-evenly',
        width: '100%',
    },
    button: {
        width: '45%',
    },
    containerLobby: {
        height: '87%',
        display: 'flex',
        justifyContent: 'space-around',
    }
});
