import {QuizInformations} from "../../../../models/QuizInformations";
import {Question} from "../../../../models/Question";
import {NavigationProp} from "@react-navigation/native";
import React, {useEffect, useState} from "react";
import {useQuizService} from "../../../../services/QuizService";
import {Answer} from "../../../../models/Answer";
import HttpError from "../../../../services/HttpError";
import AnswerButton from "../../../../components/buttons/AnswerButton";
import {ActivityIndicator, View, Text, StyleSheet, ScrollView} from "react-native";
import BlueButton from "../../../../components/buttons/BlueButton";
import {ButtonsStyles} from "../../../../styles/ButtonsStyles";
import EventSource from "react-native-sse";
import {CustomEventType} from "../../../../models/CustomEventType";
import {AnswerSSEResponse} from "../../../../models/Response/AnswerSSEResponse";
import {useMultiService} from "../../../../services/MultiService";


interface Props {
    runId?: string;
    actualQuestion?: Question;
    fetchActualQuestion: () => void;
    es: EventSource<CustomEventType>;
    roomId: string;
    isHost: boolean;
    quizState: QuizState;
    setQuizState: (quizState: QuizState) => void;
    setScore: (score: number) => void;
    navigation: NavigationProp<any>;
}

enum QuizState {
    ANSWERING= "Answering",
    LOADING= "Loading",
    SHOWING_RESULTS= "ShowingResults",
}

const getStyleOfAnswerButtonText = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingText;
    }
    if(quizState === QuizState.ANSWERING)
    {
        return styles.answerText;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctText;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectText;
    }
    return styles.answerText;
}

const getStyleOfAnswerButton = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingAnswer;
    }
    if(quizState === QuizState.ANSWERING)
    {
        if(selectedAnswerIndex === null) return styles.answerButton;
        if(selectedAnswerIndex === answerId) return styles.selectedAnswer;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctAnswer;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectAnswer;
    }
    return styles.answerButton;
}

export default function PlayingQuizMultiModeBody({runId, actualQuestion, fetchActualQuestion, roomId, es,isHost, setScore, quizState,setQuizState, navigation }: Props) {
    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);
    const [forceEnd, setForceEnd] = useState(false);
    const {revealAnswer, answerQuestion, nextQuestion, getScore} = useMultiService();
    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) return answers[0];
        return correctAnswer;
    };

    const initSocket = async () => {
        es.addEventListener("answerRevealed", (event) => {
            if(!event.data) return;
            const answers: AnswerSSEResponse = JSON.parse(event.data);
            setCorrectAnswerId(getCorrectAnswer(answers.answers).id);
            setQuizState(QuizState.SHOWING_RESULTS);
        });

        es.addEventListener("nextQuestion", (event) => {

        });

    }

    useEffect(() => {
        initSocket();
    }, []);



    const onValidation = async () => {

        if(selectedAnswerId === null || actualQuestion === undefined) return;
        setQuizState(QuizState.LOADING);
        setForceEnd(!forceEnd);
        await answerQuestion(actualQuestion.id, selectedAnswerId);

        if(isHost)
        {
            await revealAnswer();
        }

        const scoreFetched = await getScore();
        if(HttpError.isHttpError(scoreFetched))
        {
            return;
        }
        setScore(scoreFetched);

    };
    const onContinue = () => {
        setForceEnd(!forceEnd);
        if(isHost)
        {
            nextQuestion();
            setQuizState(QuizState.ANSWERING);
            setCorrectAnswerId(null);
            setSelectedAnswerId(null);
        }
    }

    const onAnsweredButtonClicked = (answerId: number) => {
        if(quizState === QuizState.SHOWING_RESULTS || quizState === QuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    }

    return (
        <View style={styles.buttonContainer}>
            {actualQuestion ? (
                <>
                <ScrollView
                    style={styles.buttonListContainer}
                    contentContainerStyle={styles.buttonListContentContainer}
                >
                    {actualQuestion.answers.map((answer, index) => (
                        <AnswerButton
                            key={index}
                            text={answer.text}
                            handleButtonPressed={() => onAnsweredButtonClicked(answer.id)}
                            buttonStyle={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                            buttonText={getStyleOfAnswerButtonText(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                            forceEnd={forceEnd}
                        />
                    ))}
                </ScrollView>
            <View style={styles.playButtonContainer}>
                <BlueButton onPress={() => quizState === QuizState.ANSWERING ? onValidation() : onContinue()} text={quizState === QuizState.ANSWERING ? "CONFIRM" : "CONTINUE"} buttonStyle={{borderRadius: 50}} isDisabled={quizState !== QuizState.ANSWERING && !isHost}/>
            </View>
                </>
            ) : (
                <View style={styles.loadingContainer}>
                    <ActivityIndicator size="large" color="#2b73fe" />
                    <Text style={styles.loadingText}>Loading...</Text>
                </View>
            )}
        </View>
    );

}

const styles = StyleSheet.create({
    buttonContainer: {
        display: 'flex',
        flex: 1,
        alignItems: 'center',
        justifyContent: 'flex-start',
        width: '100%',
        marginVertical: '3%',
    },
    loadingContainer: {
        flex: 1,
        justifyContent: "center",
        alignItems: "center",
    },
    playButtonContainer: {
        marginTop: 20, // Ajoute un espacement au-dessus du bouton Play
        width: '80%', // S'assure que le bouton prend toute la largeur disponible
        alignItems: 'center', // Centre le bouton horizontalement
    },
    correctAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'green',
        borderWidth: 2,
    },
    incorrectAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'red',
        borderWidth: 2,
    },
    loadingAnswer: {
        ...ButtonsStyles.answerButton,
        backgroundColor: '#4F6367',
        borderWidth: 2,
    },
    selectedAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'grey',
        borderWidth: 2,
    },
    answerText: {
        color: 'black',
        textAlign: 'center',
    },
    correctText: {
        fontWeight: 'bold',
        color: 'green',
    },
    incorrectText: {
        fontWeight: 'bold',
        color: 'red',
    },
    loadingText: {
        color: 'black',
        textAlign: 'center',
    },
    answerButton: {
        backgroundColor: "#F3F3F3",
        height: '75%',
        alignItems: 'center',
        width: '80%',
        marginVertical: '3%',
        justifyContent: "center",
        shadowColor: "#000",
        shadowOffset: { width: 0, height: 2 },
        shadowOpacity: 0.25,
        shadowRadius: 3.5,
        elevation: 5,
    },
    buttonListContainer: {
        maxHeight: '80%',
        width: '100%',
    },
    buttonListContentContainer: {
        display: 'flex',
        justifyContent: 'flex-start',
        alignItems: 'center'
    }
});

