import {View, StyleSheet, Text, ActivityIndicator, ScrollView} from "react-native";
import DefaultButton from "../../components/buttons/DefaultButton";
import { ButtonsStyles } from "../../styles/ButtonsStyles";
import React, { useState, useEffect, SetStateAction, Dispatch } from "react";
import { useQuizService } from "../../services/QuizService";
import {NavigationProp} from "@react-navigation/native";
import HttpError from "../../services/HttpError";
import BlueButton from "../../components/buttons/BlueButton";
import {Question} from "../../models/Question";
import {Answer} from "../../models/Answer";
import {Quiz} from "../../models/Quiz";
import AnswerButton from "../../components/buttons/AnswerButton";
import {QuizInformations} from "../../models/QuizInformations";

interface Props {
    quizInformations?: QuizInformations;
    runId?: string;
    actualQuestion: Question;
    fetchActualQuestion: () => void;
    score: number;
    setScore: (score: number) => void;
    navigation: NavigationProp<any>;
}

enum QuizState {
    ANSWERING= "Answering",
    LOADING= "Loading",
    SHOWING_RESULTS= "ShowingResults",
}

const getStyleOfAnswerButtonText = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingText;
    }
    if(quizState === QuizState.ANSWERING)
    {
        return styles.answerText;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctText;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectText;
    }
    return styles.answerText;
}

const getStyleOfAnswerButton = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingAnswer;
    }
    if(quizState === QuizState.ANSWERING)
    {
        if(selectedAnswerIndex === null) return styles.answerButton;
        if(selectedAnswerIndex === answerId) return styles.selectedAnswer;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctAnswer;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectAnswer;
    }
    return styles.answerButton;
}

export default function PlayingQuizBody({ quizInformations, runId, actualQuestion, fetchActualQuestion, score, setScore, navigation }: Props) {
    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [quizState, setQuizState] = useState(QuizState.ANSWERING);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);
    const [isLastQuestion, setIsLastQuestion] = useState(false);
    const [forceEnd, setForceEnd] = useState(false);

    const {answerQuestion} = useQuizService();
    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) throw new Error("No correct answer found");
        return correctAnswer;
    };
    const onValidation = async () => {
        if(selectedAnswerId === null) return;
        setQuizState(QuizState.LOADING);
        setForceEnd(!forceEnd);
        const answerId = actualQuestion.answers.find((answer) => answer.id === selectedAnswerId)?.id;
        if(!answerId || !runId) return;
        const answereFetched = await answerQuestion(runId, actualQuestion.id, answerId);

        if (HttpError.isHttpError(answereFetched)) {
            return;
        }
        setScore(answereFetched.score);
        const correctAnswerIdFetched = getCorrectAnswer(answereFetched.answers.answers).id;
        setCorrectAnswerId(correctAnswerIdFetched);
        setQuizState(QuizState.SHOWING_RESULTS);

        if (quizInformations === undefined) return;
        // Check si c'est la dernière question
        if(answereFetched.answers.order === quizInformations.questionCount - 1) {
            setIsLastQuestion(true);
            return;
        }

    };
    const onContinue = async () => {
        await setForceEnd(!forceEnd);
        if(isLastQuestion) {
            navigation.reset({
                index: 0,
                routes: [
                    {
                        name: "EndQuiz",
                        params: {quizInformations: quizInformations, score: score},
                    },
                ]
            });
            return;
        }
        fetchActualQuestion();
        setQuizState(QuizState.ANSWERING);
        setSelectedAnswerId(null);
        setCorrectAnswerId(null);
    }

    const onAnsweredButtonClicked = (answerId: number) => {
        if(quizState === QuizState.SHOWING_RESULTS || quizState === QuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    }

    return (
        <View style={styles.buttonContainer}>
            {actualQuestion ? (
                <>
                    <ScrollView 
                        style={styles.buttonListContainer}
                        contentContainerStyle={styles.buttonListContentContainer}
                    >
                        {actualQuestion.answers.map((answer, index) => (
                            <AnswerButton
                                key={index}
                                text={answer.text}
                                handleButtonPressed={() => onAnsweredButtonClicked(answer.id)}
                                buttonStyle={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                                buttonText={getStyleOfAnswerButtonText(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                                forceEnd={forceEnd}
                            />
                        ))}
                    </ScrollView>
                    <View style={styles.playButtonContainer}>
                        <BlueButton onPress={() => quizState === QuizState.ANSWERING ? onValidation() : onContinue()} text={quizState === QuizState.ANSWERING ? "CONFIRM" : "CONTINUE"} buttonStyle={{borderRadius: 50}} isDisabled={false}/>
                    </View>
                </>
            ) : (
                <View style={styles.loadingContainer}>
                    <ActivityIndicator size="large" color="#2b73fe" />
                    <Text style={styles.loadingText}>Loading...</Text>
                </View>
            )}
        </View>
    );

}

const styles = StyleSheet.create({
    buttonContainer: {
        display: 'flex',
        flex: 1,
        alignItems: 'center',
        justifyContent: 'flex-start',
        width: '100%',
        marginVertical: '3%',
    },
    loadingContainer: {
        flex: 1,
        justifyContent: "center",
        alignItems: "center",
    },
    playButtonContainer: {
        marginTop: 20, // Ajoute un espacement au-dessus du bouton Play
        width: '80%', // S'assure que le bouton prend toute la largeur disponible
        alignItems: 'center', // Centre le bouton horizontalement
    },
    correctAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'green',
        borderWidth: 2,
    },
    incorrectAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'red',
        borderWidth: 2,
    },
    loadingAnswer: {
        ...ButtonsStyles.answerButton,
        backgroundColor: '#4F6367',
        borderWidth: 2,
    },
    selectedAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'grey',
        borderWidth: 2,
    },
    answerText: {
        color: 'black',
        textAlign: 'center',
    },
    correctText: {
        fontWeight: 'bold',
        color: 'green',
    },
    incorrectText: {
        fontWeight: 'bold',
        color: 'red',
    },
    loadingText: {
        color: 'black',
        textAlign: 'center',
    },
    answerButton: {
        backgroundColor: "#F3F3F3",
        height: '75%',
        alignItems: 'center',
        width: '80%',
        marginVertical: '3%',
        justifyContent: "center",
        shadowColor: "#000",
        shadowOffset: { width: 0, height: 2 },
        shadowOpacity: 0.25,
        shadowRadius: 3.5,
        elevation: 5,
    },
    buttonListContainer: {
        maxHeight: '80%',
        width: '100%',
    },
    buttonListContentContainer: {
        display: 'flex',
        justifyContent: 'flex-start',
        alignItems: 'center'
    }
});

